#include "ReadFluka.h"
#include <iostream>

#define READ_ERROR 1
#define CANT_OPEN_FILE 2
#define FILENAME_EMPTY 3

using namespace std;

ReadFluka::ReadFluka(const char *fname)
{
  if (fname == 0) {
    cerr << "input file name is not specified" << endl;
    exit(FILENAME_EMPTY);
  }
  fin = new ifstream(fname);
  if (!fin->is_open()) {
    cerr << "can't open input file " << fname << endl;
    exit(CANT_OPEN_FILE);
  }
  fchar     = new Char_t[81];
  fRunTitle = new Char_t[81];
  fRunTime = new Char_t[32];
  fVerbose = 1;
}

ReadFluka::~ReadFluka()
{
  delete [] fRunTime;  fRunTime = 0;
  delete [] fRunTitle; fRunTitle = 0;
  delete [] fchar;     fchar = 0;
  if (fin) {
    fin->close();
    delete fin;
    fin = 0;
  }
}

Int_t ReadFluka::ReadInt(Int_t n/* =1 */) const
{
  // reading n integers
  Int_t data[n];
  fin->read((Char_t *)data, sizeof(Int_t)*n);
  if (!fin->good()) {
    if (fVerbose>2) cerr << "read error in ReadFluka::ReadInt()" << endl;
  };
  return data[0];
}
 
Float_t ReadFluka::ReadFloat(Int_t n/* =1 */) const
{
  // reading n doubles
  Float_t data[n];
  fin->read((Char_t *)data, sizeof(Float_t)*n);
  if (!fin->good()) {
    if (fVerbose>2) cerr << "read error in ReadFluka::ReadFloat()" << endl;
  };
  return data[0];
}

Bool_t ReadFluka::ReadBool(Int_t n/* =1 */) const
{
  // read n bools
  // since we're reading FORTRAN binary output, sizeof(bool) = sizeof(int)
  // => read n ints:
  return (Bool_t)ReadInt(n);

  /*
  Bool_t data[n];
  fin->read((Char_t *)data, sizeof(Bool_t)*n);
  if (!fin->good()) {
    if (fVerbose>2) cerr << "read error in ReadFluka::ReadBool()" << endl;
  };
  return data[0];
  */
}

void ReadFluka::ReadRunTitle()
{
  ReadInt();
  fin->read(fchar, 80);
  if (!fin->good()) {
    if (fVerbose>1) cerr << "read error in ReadFluka::ReadRunTitle()" << endl;
  };
  fchar[80] = '\0';
  strcpy(fRunTitle, fchar);  
  if (fVerbose) cout << "title:\t" << fRunTitle << endl;
}

void ReadFluka::ReadRunTime()
{
  fin->read(fchar, 32);
  if (!fin->good()) {
    if (fVerbose>1) cerr << "read error in ReadFluka::ReadRunTime()" << endl;
  };
  fchar[32] = '\0';
  strcpy(fRunTime, fchar);
  if (fVerbose) cout << "time:\t" << fRunTime << endl;
}
