#include <iostream>
#include "UsrYield.h"
#include <TString.h>
#include <cmath>
using namespace std;

UsrYield::UsrYield(const char *fname) : ReadFluka(fname)
{
  fIsReadHeader = false;
  fWEIPRI = 0;
  fNCASE  = 0;
  fIJUSYL = fJTUSYL = 0;
  fPUSRYL = fSQSUYL = 0.0;
  fUUSRYL = fVUSRYL = fWUSRYL = 0.0;
  fTITUYL = new Char_t[11];
  fSCORED = 0;
  fENGMAX = 0;

  Reset();
  ReadHeader();
}

void UsrYield::Reset()
{
  fMY = fITUSYL = fIXUSYL = fIDUSYL = fNR1UYL = fNR2UYL = 0;
  fUSNRYL = fSGUSYL = 0.0;
  fLLNUYL = false;
  //  for (int i=0; i<10; i++) fTITUYL[i] = "";
  fEYLLOW = fEYLHGH = 0.0;
  fNEYLBN = 0;
  fDEYLBN = 0.0;
  fAYLLOW = fAYLHGH = 0;
  fIGMUYL = 0;
  if (fSCORED) { delete [] fSCORED; fSCORED = 0;}
  if (fENGMAX) { delete [] fENGMAX; fENGMAX = 0;}
  fINTERV = 0;
  fie = fia = 0;
  fixa = fixm = 0;
}



UsrYield::~UsrYield()
{
  Reset();
  delete [] fTITUYL; fTITUYL = 0;
}

void UsrYield::ReadHeader()
{
  if (fIsReadHeader == false) { // did not read the header yet?
    fIsReadHeader = true;
    
    
    ReadRunTitle();
    ReadRunTime();
    
    fWEIPRI = ReadFloat();      
    fNCASE  = ReadInt(3);      
    fIJUSYL = ReadInt();       
    fJTUSYL = ReadInt();       
    fPUSRYL = ReadFloat();    
    fSQSUYL = ReadFloat();     
    fUUSRYL = ReadFloat();     
    fVUSRYL = ReadFloat();    
    fWUSRYL = ReadFloat();   
    
    if (fVerbose) {
      cout << "WEIPRI:\t" << fWEIPRI << endl;
      cout << "NCASE:\t" << fNCASE << endl;
      cout << "IJYSUL:\t" << fIJUSYL << endl;
      cout << "JTUSYL:\t" << fJTUSYL << endl;
      cout << "PUSRYL:\t" << fPUSRYL << endl;
      cout << "SQSUYLL:\t" << fSQSUYL << endl;
      cout << "UUSRYL:\t" << fUUSRYL << endl;
      cout << "VUSRYL:\t" << fVUSRYL << endl;
      cout << "WUSRYL:\t" << fWUSRYL << endl;
    }
  }
}

Bool_t UsrYield::Read()
{
  Reset();
  TString str;
  ReadFloat(2);
  if (!fin->good()) return false;
  for (Int_t i=0; i<1; i++) {
    fMY = ReadInt();
    fin->read(fTITUYL, 10);
    fTITUYL[10] = '\0';
    str = fTITUYL;
    str = str.Strip(TString::kBoth);
    strcpy(fTITUYL, str.Data());
    
    fITUSYL = ReadInt();
    fie = fITUSYL % 100;
    fia = (fITUSYL-fie) / 100;
    
    fIXUSYL = ReadInt();
    fixa = fIXUSYL % 100;
    fixm = (fIXUSYL-fixa) / 100;
    
    fIDUSYL = ReadInt();
    fNR1UYL = ReadInt();
    fNR2UYL = ReadInt();
    fUSNRYL = ReadFloat();
    fSGUSYL = ReadFloat();
    fLLNUYL = ReadBool();
    fEYLLOW = ReadFloat();
    fEYLHGH = ReadFloat();
    fNEYLBN = ReadInt();
    fDEYLBN = ReadFloat();
    fAYLLOW = ReadFloat();
    fAYLHGH = ReadFloat();
    
    ReadInt();
    
    if (fVerbose) {
      cout << "MY:\t" << fMY << endl;
      cout << "TITUYL:\t" << fTITUYL << endl;
      cout << "ITUSYL:\t" << fITUSYL << endl;
      cout << "IXUSYL:\t" << fIXUSYL << endl;
      cout << "IDUSYL:\t" << fIDUSYL << endl;
      cout << "from region " << fNR1UYL << " to region " << fNR2UYL << endl;
      cout << "user normalization factor:\t" << fUSNRYL << endl;
      cout << "adopted cross section:\t" << fSGUSYL << endl;
      cout << fLLNUYL << endl;

      cout << "first variable: from " << fEYLLOW;
      if (fITUSYL % 100 <=2 )	cout << " GeV,";
      else if (fITUSYL % 100 == 14) cout << " rad, ";
      cout << " to " << fEYLHGH;
      if (fITUSYL % 100 <=2 ) cout << " GeV,";
      else if (fITUSYL % 100 == 14) cout << " rad, ";
      cout << " " << fNEYLBN << " bins (";
      cout << fDEYLBN <<" GeV wide)"<< endl;
      cout << "angle from " << fAYLLOW << " to " << fAYLHGH << " ster" << endl;
    }
    Double_t AAUSYL = 0.01L * fITUSYL;
    Int_t IAUSYL=0;

    // NINT
    if (AAUSYL >= 0) {
      IAUSYL = int(AAUSYL + 0.5);
      if (AAUSYL + 0.5 == Float_t(IAUSYL) && IAUSYL & 1) IAUSYL--;
    } else {
      IAUSYL = int(AAUSYL - 0.5);
      if (AAUSYL - 0.5 == Float_t(IAUSYL) && IAUSYL & 1) IAUSYL++;
    }
    //
    //    Int_t IEUSYL = fITUSYL - IAUSYL*100;
    
    if (fLLNUYL){
      ReadInt();
      // if low-energy neutrons, read group energies
      if (fVerbose) cout << "low-energy neutrons requested" << endl;
      fIGMUYL = ReadInt();
      if (fVerbose>2) cout << "ENGMAX[" << fIGMUYL+1 << "]:\t";
      fENGMAX = new Float_t[fIGMUYL+1];
      for (Int_t i=0; i<fIGMUYL+1; i++) {
	fENGMAX[i] = ReadFloat();
	if (fVerbose>2) cout << fENGMAX[i] << ' ';
      }
      ReadFloat();
      if (fVerbose>2) cout << endl;
    } else fIGMUYL = 0;
    
    //  ----- if second variable is angle, differential is in sr ------
    Double_t TTLOW=0L, TTHGH=0L, DAYLBN=0L;
    if ( (IAUSYL == 14) || (IAUSYL == 15) ) {
      if (fAYLHGH < 0.3L * M_PI) {
	// small angle (exact!)
	TTLOW = tan(fAYLLOW/2.0L);
	TTHGH = tan(fAYLHGH/2.0L);
	DAYLBN = M_PI*2.0L * 2.0L * ( TTHGH + TTLOW ) * ( TTHGH - TTLOW ) / ( 1.0L + pow(TTLOW, 2) ) / ( 1.0L + pow(TTHGH, 2) );
      } else {
	// large angle:
	DAYLBN = M_PI*2.0L * ( cos((Double_t)fAYLLOW) - cos((Double_t)fAYLHGH) );
      }
    } else {
      DAYLBN = (Double_t)fAYLHGH - (Double_t)fAYLLOW;
    }
    
    fINTERV = fNEYLBN + fIGMUYL;
    cout << "number of scored values: " << fNEYLBN << endl;
    fSCORED = new Float_t[fNEYLBN];
    
    cout << "scored[" << fNEYLBN << "]:\t";
    ReadFloat();
    for (int i=0; i<fNEYLBN; i++) {
      fSCORED[i] = ReadFloat();
      cout.setf(ios::scientific);
      cout << fSCORED[i] << ' ';
    }
    ReadFloat(fIGMUYL); // read these zeros
    cout << endl;
    
    //    if (IEUSYL > 0)

  }
  
  MakeHist();
  
  return true;
}

const Char_t *UsrYield::GetXTitle() const {
  string x1("x");
  
  if (fie<0) x1 = "log ";
  else x1 = "";
  
  switch (abs(fie)) {
  case (5): x1 += "#eta_{lab}";
    break;
  case (6): x1 += "#eta_{cms}";
    break;
  case (14): x1 += "#theta_{lab}";
    break;
  }
  
  return x1.c_str();
}

const Char_t *UsrYield::GetYTitle() const {
  string y("y");
  string x2;
  
  switch(abs(fia)) {
  case(1):
    x2 = "E_{kin}";
    break;
  }
  
  switch (fixa) {
  case(3): 
    if (x2.length())
      y = "#frac{d^{2}N}{d";
    else y = "#frac{dN}{d";
    y += GetXTitle();
    if (x2.length()) {
      y += "d";
      y += x2;
    }
    y += "}";
    break;
  }
  return y.c_str();
}

const Char_t *UsrYield::GetNameOfScoredParticles() const {
  string name = "";
  //  sprintf(name, "%d", GetTypeOfScoredParticles());
  
  switch(GetTypeOfScoredParticles()) {
  case(202):
    name = "ALL_CHAR";
    break;
  }
  return name.c_str();
}

void UsrYield::MakeHist()
{
  TH1F *h = new TH1F(fTITUYL, "", fNEYLBN, fEYLLOW, fEYLHGH);
  for (Int_t i=0; i<fNEYLBN; i++)
    h->SetBinContent(i+1, fSCORED[i]);
  
  fHists.Add(h->Clone());
  delete h; h=0;
}
